# Copyright 2017 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the &quot;License&quot;);
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an &quot;AS IS&quot; BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# NOTE: This class is auto generated by the swagger code generator program.
# https://github.com/swagger-api/swagger-codegen.git
# Do not edit the class manually.

defmodule GoogleApi.AppState.V1.Api.States do
  @moduledoc """
  API calls for all endpoints tagged `States`.
  """

  alias GoogleApi.AppState.V1.Connection
  alias GoogleApi.Gax.{Request, Response}

  @doc """
  Clears (sets to empty) the data for the passed key if and only if the passed version matches the currently stored version. This method results in a conflict error on version mismatch.

  ## Parameters

  - connection (GoogleApi.AppState.V1.Connection): Connection to server
  - state_key (integer()): The key for the data to be retrieved.
  - opts (KeywordList): [optional] Optional parameters
    - :alt (String.t): Data format for the response.
    - :fields (String.t): Selector specifying which fields to include in a partial response.
    - :key (String.t): API key. Your API key identifies your project and provides you with API access, quota, and reports. Required unless you provide an OAuth 2.0 token.
    - :oauth_token (String.t): OAuth 2.0 token for the current user.
    - :prettyPrint (boolean()): Returns response with indentations and line breaks.
    - :quotaUser (String.t): An opaque string that represents a user for quota purposes. Must not exceed 40 characters.
    - :userIp (String.t): Deprecated. Please use quotaUser instead.
    - :currentDataVersion (String.t): The version of the data to be cleared. Version strings are returned by the server.

  ## Returns

  {:ok, %GoogleApi.AppState.V1.Model.WriteResult{}} on success
  {:error, info} on failure
  """
  @spec appstate_states_clear(Tesla.Env.client(), integer(), keyword()) ::
          {:ok, GoogleApi.AppState.V1.Model.WriteResult.t()} | {:error, Tesla.Env.t()}
  def appstate_states_clear(connection, state_key, opts \\ []) do
    optional_params = %{
      :alt => :query,
      :fields => :query,
      :key => :query,
      :oauth_token => :query,
      :prettyPrint => :query,
      :quotaUser => :query,
      :userIp => :query,
      :currentDataVersion => :query
    }

    request =
      Request.new()
      |> Request.method(:post)
      |> Request.url("/states/{stateKey}/clear", %{
        "stateKey" => state_key
      })
      |> Request.add_optional_params(optional_params, opts)

    connection
    |> Connection.execute(request)
    |> Response.decode(struct: %GoogleApi.AppState.V1.Model.WriteResult{})
  end

  @doc """
  Deletes a key and the data associated with it. The key is removed and no longer counts against the key quota. Note that since this method is not safe in the face of concurrent modifications, it should only be used for development and testing purposes. Invoking this method in shipping code can result in data loss and data corruption.

  ## Parameters

  - connection (GoogleApi.AppState.V1.Connection): Connection to server
  - state_key (integer()): The key for the data to be retrieved.
  - opts (KeywordList): [optional] Optional parameters
    - :alt (String.t): Data format for the response.
    - :fields (String.t): Selector specifying which fields to include in a partial response.
    - :key (String.t): API key. Your API key identifies your project and provides you with API access, quota, and reports. Required unless you provide an OAuth 2.0 token.
    - :oauth_token (String.t): OAuth 2.0 token for the current user.
    - :prettyPrint (boolean()): Returns response with indentations and line breaks.
    - :quotaUser (String.t): An opaque string that represents a user for quota purposes. Must not exceed 40 characters.
    - :userIp (String.t): Deprecated. Please use quotaUser instead.

  ## Returns

  {:ok, %{}} on success
  {:error, info} on failure
  """
  @spec appstate_states_delete(Tesla.Env.client(), integer(), keyword()) ::
          {:ok, nil} | {:error, Tesla.Env.t()}
  def appstate_states_delete(connection, state_key, opts \\ []) do
    optional_params = %{
      :alt => :query,
      :fields => :query,
      :key => :query,
      :oauth_token => :query,
      :prettyPrint => :query,
      :quotaUser => :query,
      :userIp => :query
    }

    request =
      Request.new()
      |> Request.method(:delete)
      |> Request.url("/states/{stateKey}", %{
        "stateKey" => state_key
      })
      |> Request.add_optional_params(optional_params, opts)

    connection
    |> Connection.execute(request)
    |> Response.decode(decode: false)
  end

  @doc """
  Retrieves the data corresponding to the passed key. If the key does not exist on the server, an HTTP 404 will be returned.

  ## Parameters

  - connection (GoogleApi.AppState.V1.Connection): Connection to server
  - state_key (integer()): The key for the data to be retrieved.
  - opts (KeywordList): [optional] Optional parameters
    - :alt (String.t): Data format for the response.
    - :fields (String.t): Selector specifying which fields to include in a partial response.
    - :key (String.t): API key. Your API key identifies your project and provides you with API access, quota, and reports. Required unless you provide an OAuth 2.0 token.
    - :oauth_token (String.t): OAuth 2.0 token for the current user.
    - :prettyPrint (boolean()): Returns response with indentations and line breaks.
    - :quotaUser (String.t): An opaque string that represents a user for quota purposes. Must not exceed 40 characters.
    - :userIp (String.t): Deprecated. Please use quotaUser instead.

  ## Returns

  {:ok, %GoogleApi.AppState.V1.Model.GetResponse{}} on success
  {:error, info} on failure
  """
  @spec appstate_states_get(Tesla.Env.client(), integer(), keyword()) ::
          {:ok, GoogleApi.AppState.V1.Model.GetResponse.t()} | {:error, Tesla.Env.t()}
  def appstate_states_get(connection, state_key, opts \\ []) do
    optional_params = %{
      :alt => :query,
      :fields => :query,
      :key => :query,
      :oauth_token => :query,
      :prettyPrint => :query,
      :quotaUser => :query,
      :userIp => :query
    }

    request =
      Request.new()
      |> Request.method(:get)
      |> Request.url("/states/{stateKey}", %{
        "stateKey" => state_key
      })
      |> Request.add_optional_params(optional_params, opts)

    connection
    |> Connection.execute(request)
    |> Response.decode(struct: %GoogleApi.AppState.V1.Model.GetResponse{})
  end

  @doc """
  Lists all the states keys, and optionally the state data.

  ## Parameters

  - connection (GoogleApi.AppState.V1.Connection): Connection to server
  - opts (KeywordList): [optional] Optional parameters
    - :alt (String.t): Data format for the response.
    - :fields (String.t): Selector specifying which fields to include in a partial response.
    - :key (String.t): API key. Your API key identifies your project and provides you with API access, quota, and reports. Required unless you provide an OAuth 2.0 token.
    - :oauth_token (String.t): OAuth 2.0 token for the current user.
    - :prettyPrint (boolean()): Returns response with indentations and line breaks.
    - :quotaUser (String.t): An opaque string that represents a user for quota purposes. Must not exceed 40 characters.
    - :userIp (String.t): Deprecated. Please use quotaUser instead.
    - :includeData (boolean()): Whether to include the full data in addition to the version number

  ## Returns

  {:ok, %GoogleApi.AppState.V1.Model.ListResponse{}} on success
  {:error, info} on failure
  """
  @spec appstate_states_list(Tesla.Env.client(), keyword()) ::
          {:ok, GoogleApi.AppState.V1.Model.ListResponse.t()} | {:error, Tesla.Env.t()}
  def appstate_states_list(connection, opts \\ []) do
    optional_params = %{
      :alt => :query,
      :fields => :query,
      :key => :query,
      :oauth_token => :query,
      :prettyPrint => :query,
      :quotaUser => :query,
      :userIp => :query,
      :includeData => :query
    }

    request =
      Request.new()
      |> Request.method(:get)
      |> Request.url("/states")
      |> Request.add_optional_params(optional_params, opts)

    connection
    |> Connection.execute(request)
    |> Response.decode(struct: %GoogleApi.AppState.V1.Model.ListResponse{})
  end

  @doc """
  Update the data associated with the input key if and only if the passed version matches the currently stored version. This method is safe in the face of concurrent writes. Maximum per-key size is 128KB.

  ## Parameters

  - connection (GoogleApi.AppState.V1.Connection): Connection to server
  - state_key (integer()): The key for the data to be retrieved.
  - opts (KeywordList): [optional] Optional parameters
    - :alt (String.t): Data format for the response.
    - :fields (String.t): Selector specifying which fields to include in a partial response.
    - :key (String.t): API key. Your API key identifies your project and provides you with API access, quota, and reports. Required unless you provide an OAuth 2.0 token.
    - :oauth_token (String.t): OAuth 2.0 token for the current user.
    - :prettyPrint (boolean()): Returns response with indentations and line breaks.
    - :quotaUser (String.t): An opaque string that represents a user for quota purposes. Must not exceed 40 characters.
    - :userIp (String.t): Deprecated. Please use quotaUser instead.
    - :currentStateVersion (String.t): The version of the app state your application is attempting to update. If this does not match the current version, this method will return a conflict error. If there is no data stored on the server for this key, the update will succeed irrespective of the value of this parameter.
    - :body (UpdateRequest): 

  ## Returns

  {:ok, %GoogleApi.AppState.V1.Model.WriteResult{}} on success
  {:error, info} on failure
  """
  @spec appstate_states_update(Tesla.Env.client(), integer(), keyword()) ::
          {:ok, GoogleApi.AppState.V1.Model.WriteResult.t()} | {:error, Tesla.Env.t()}
  def appstate_states_update(connection, state_key, opts \\ []) do
    optional_params = %{
      :alt => :query,
      :fields => :query,
      :key => :query,
      :oauth_token => :query,
      :prettyPrint => :query,
      :quotaUser => :query,
      :userIp => :query,
      :currentStateVersion => :query,
      :body => :body
    }

    request =
      Request.new()
      |> Request.method(:put)
      |> Request.url("/states/{stateKey}", %{
        "stateKey" => state_key
      })
      |> Request.add_optional_params(optional_params, opts)

    connection
    |> Connection.execute(request)
    |> Response.decode(struct: %GoogleApi.AppState.V1.Model.WriteResult{})
  end
end
